@@ -33,6 +33,7 @@ typedef enum {
   GDK_GL_FEATURE_VERTEX_HALF_FLOAT          = 1 << 2,
   GDK_GL_FEATURE_SYNC                       = 1 << 3,
   GDK_GL_FEATURE_BASE_INSTANCE              = 1 << 4,
+  GDK_GL_FEATURE_BUFFER_STORAGE             = 1 << 5,
 } GdkGLFeatures;
 
 typedef enum {
@@ -121,8 +122,9 @@ void                    gdk_gl_backend_use                      (GdkGLBackend
 
 void                    gdk_gl_context_clear_current_if_surface (GdkSurface      *surface);
 
-GdkGLContext *          gdk_gl_context_new                      (GdkDisplay      *display,
-                                                                 GdkSurface      *surface);
+GdkGLContext *          gdk_gl_context_new                      (GdkDisplay             *display,
+                                                                 GdkSurface             *surface,
+                                                                 gboolean                surface_attached);
 
 gboolean                gdk_gl_context_is_api_allowed           (GdkGLContext           *self,
                                                                  GdkGLAPI                api,
