@@ -1160,6 +1160,8 @@ gdk_surface_get_egl_surface (GdkSurface *self)
 {
   GdkSurfacePrivate *priv = gdk_surface_get_instance_private (self);
 
+  gdk_surface_ensure_egl_surface (self, priv->egl_surface_depth);
+
   return priv->egl_surface;
 }
 
@@ -1172,6 +1174,14 @@ gdk_surface_ensure_egl_surface (GdkSurface     *self,
 
   g_return_val_if_fail (priv->egl_native_window != NULL, depth);
 
+  if (depth == GDK_MEMORY_NONE)
+    {
+      if (priv->egl_surface_depth == GDK_MEMORY_NONE)
+        depth = GDK_MEMORY_U8;
+      else
+        depth = priv->egl_surface_depth;
+    }
+
   if (priv->egl_surface_depth != depth &&
       priv->egl_surface != NULL &&
       gdk_display_get_egl_config (display, priv->egl_surface_depth) != gdk_display_get_egl_config (display, depth))
@@ -2762,7 +2772,8 @@ gdk_surface_is_opaque (GdkSurface *self)
   if (gdk_rectangle_contains (&priv->opaque_rect, &whole))
     return TRUE;
 
-  if (cairo_region_contains_rectangle (priv->opaque_region, &whole) == CAIRO_REGION_OVERLAP_IN)
+  if (priv->opaque_region &&
+      cairo_region_contains_rectangle (priv->opaque_region, &whole) == CAIRO_REGION_OVERLAP_IN)
     return TRUE;
 
   return FALSE;
