@@ -56,7 +56,6 @@
 
 #include "gsk/gskrectprivate.h"
 
-
 /**
  * GdkWaylandSurface:
  *
@@ -773,6 +772,20 @@ gdk_wayland_surface_sync_viewport (GdkSurface *surface)
   self->viewport_dirty = FALSE;
 }
 
+static void
+gdk_wayland_surface_sync_color_state (GdkSurface *surface)
+{
+  GdkWaylandSurface *self = GDK_WAYLAND_SURFACE (surface);
+
+  if (!self->color_state_changed)
+    return;
+
+  gdk_wayland_color_surface_set_color_state (self->display_server.color,
+                                             gdk_surface_get_color_state (surface));
+
+  self->color_state_changed = FALSE;
+}
+
 void
 gdk_wayland_surface_sync (GdkSurface *surface)
 {
@@ -780,6 +793,7 @@ gdk_wayland_surface_sync (GdkSurface *surface)
   gdk_wayland_surface_sync_opaque_region (surface);
   gdk_wayland_surface_sync_input_region (surface);
   gdk_wayland_surface_sync_buffer_scale (surface);
+  gdk_wayland_surface_sync_color_state (surface);
   gdk_wayland_surface_sync_viewport (surface);
 }
 
@@ -792,6 +806,7 @@ gdk_wayland_surface_needs_commit (GdkSurface *surface)
          self->opaque_region_dirty ||
          self->input_region_dirty ||
          self->buffer_scale_dirty ||
+         self->color_state_changed ||
          self->viewport_dirty;
 }
 
@@ -893,13 +908,10 @@ surface_preferred_buffer_transform (void              *data,
                                     uint32_t           transform)
 {
   GdkSurface *surface = GDK_SURFACE (data);
-  const char *transform_name[] = {
-    "normal", "90", "180", "270", "flipped", "flipped-90", "flipped-180", "flipped-270"
-  };
 
   GDK_DISPLAY_DEBUG (gdk_surface_get_display (surface), EVENTS,
                      "preferred buffer transform, surface %p transform %s",
-                     surface, transform_name[transform]);
+                     surface, gdk_dihedral_get_name ((GdkDihedral) transform));
 }
 
 static const struct wl_surface_listener surface_listener = {
@@ -909,6 +921,18 @@ static const struct wl_surface_listener surface_listener = {
   surface_preferred_buffer_transform,
 };
 
+static void
+preferred_changed (GdkWaylandColorSurface *color,
+                   GdkColorState          *color_state,
+                   gpointer                data)
+{
+  GdkWaylandSurface *self = GDK_WAYLAND_SURFACE (data);
+
+  gdk_surface_set_color_state (GDK_SURFACE (self), color_state);
+
+  self->color_state_changed = TRUE;
+}
+
 static void
 gdk_wayland_surface_create_wl_surface (GdkSurface *surface)
 {
@@ -933,6 +957,12 @@ gdk_wayland_surface_create_wl_surface (GdkSurface *surface)
           wp_viewporter_get_viewport (display_wayland->viewporter, wl_surface);
     }
 
+  if (display_wayland->color)
+    self->display_server.color = gdk_wayland_color_surface_new (display_wayland->color,
+                                                                wl_surface,
+                                                                preferred_changed,
+                                                                self);
+
   self->display_server.wl_surface = wl_surface;
 }
 
@@ -986,6 +1016,7 @@ gdk_wayland_surface_destroy_wl_surface (GdkWaylandSurface *self)
 
   g_clear_pointer (&self->display_server.viewport, wp_viewport_destroy);
   g_clear_pointer (&self->display_server.fractional_scale, wp_fractional_scale_v1_destroy);
+  g_clear_pointer (&self->display_server.color, gdk_wayland_color_surface_free);
 
   g_clear_pointer (&self->display_server.wl_surface, wl_surface_destroy);
 
