@@ -134,7 +134,7 @@ struct _GdkWin32Display
   guint hasWglEXTSwapControl : 1;
   guint hasWglOMLSyncControl : 1;
   guint hasWglARBPixelFormat : 1;
-  guint force_enable_depth_bits : 1;
+  guint hasGlWINSwapHint : 1;
 
 #ifdef HAVE_EGL
   guint hasEglKHRCreateContext : 1;
