@@ -260,48 +260,64 @@ gdk_x11_cursor_create_for_name (GdkDisplay  *display,
 
 /**
  * gdk_x11_display_set_cursor_theme:
- * @display: (type GdkX11Display): a #GdkDisplay
- * @theme: (nullable): the name of the cursor theme to use, or %NULL to unset
- *         a previously set value
+ * @display: (type GdkX11Display): a `GdkDisplay`
+ * @theme: (nullable): the name of the cursor theme to use, or %NULL
+ *   to unset a previously set value
  * @size: the cursor size to use, or 0 to keep the previous size
  *
  * Sets the cursor theme from which the images for cursor
  * should be taken.
  *
  * If the windowing system supports it, existing cursors created
- * with gdk_cursor_new_from_name() are updated to reflect the theme
- * change. Custom cursors constructed with gdk_cursor_new_from_texture()
+ * with [ctor@Gdk.Cursor.new_from_name] are updated to reflect the theme
+ * change. Custom cursors constructed with [ctor@Gdk.Cursor.new_from_texture]
  * will have to be handled by the application (GTK applications can learn
  * about cursor theme changes by listening for change notification
- * for the corresponding #GtkSetting).
+ * for the corresponding `GtkSetting`).
+ *
+ * Deprecated: 4.16: Use the cursor-related properties of
+ *   [GtkSettings](../gtk4/class.Settings.html) to set the cursor theme
  */
 void
 gdk_x11_display_set_cursor_theme (GdkDisplay  *display,
                                   const char *theme,
                                   const int    size)
+{
+  _gdk_x11_display_set_cursor_theme (display, theme, size);
+}
+
+void
+_gdk_x11_display_set_cursor_theme (GdkDisplay  *display,
+                                   const char *theme,
+                                   const int    size)
 {
 #if defined(HAVE_XCURSOR) && defined(HAVE_XFIXES) && XFIXES_MAJOR >= 2
+  GdkX11Screen *x11_screen;
   Display *xdisplay;
   char *old_theme;
+  int real_size;
   int old_size;
   gpointer cursor, xcursor;
   GHashTableIter iter;
 
   g_return_if_fail (GDK_IS_DISPLAY (display));
 
+  x11_screen = gdk_x11_display_get_screen (display);
   xdisplay = GDK_DISPLAY_XDISPLAY (display);
 
+  real_size = size * x11_screen->surface_scale;
+
   old_theme = XcursorGetTheme (xdisplay);
   old_size = XcursorGetDefaultSize (xdisplay);
 
-  if (old_size == size &&
+  if (old_size == real_size &&
       (old_theme == theme ||
        (old_theme && theme && strcmp (old_theme, theme) == 0)))
     return;
 
   XcursorSetTheme (xdisplay, theme);
-  if (size > 0)
-    XcursorSetDefaultSize (xdisplay, size);
+  if (real_size > 0)
+    XcursorSetDefaultSize (xdisplay, real_size);
 
   if (GDK_X11_DISPLAY (display)->cursors == NULL)
     return;
@@ -331,19 +347,19 @@ gdk_x11_display_set_cursor_theme (GdkDisplay  *display,
 
 /**
  * gdk_x11_display_get_xcursor:
- * @display: (type GdkX11Display): a #GdkDisplay
- * @cursor: a #GdkCursor.
- * 
- * Returns the X cursor belonging to a #GdkCursor, potentially
+ * @display: (type GdkX11Display): a `GdkDisplay`
+ * @cursor: a `GdkCursor`
+ *
+ * Returns the X cursor belonging to a `GdkCursor`, potentially
  * creating the cursor.
  *
  * Be aware that the returned cursor may not be unique to @cursor.
  * It may for example be shared with its fallback cursor. On old
  * X servers that don't support the XCursor extension, all cursors
  * may even fall back to a few default cursors.
- * 
+ *
  * Returns: an Xlib Cursor.
- **/
+ */
 Cursor
 gdk_x11_display_get_xcursor (GdkDisplay *display,
                              GdkCursor  *cursor)
@@ -365,11 +381,33 @@ gdk_x11_display_get_xcursor (GdkDisplay *display,
 
   if (gdk_cursor_get_name (cursor))
     xcursor = gdk_x11_cursor_create_for_name (display, gdk_cursor_get_name (cursor));
-  else
+  else if (gdk_cursor_get_texture (cursor))
     xcursor = gdk_x11_cursor_create_for_texture (display,
                                                  gdk_cursor_get_texture (cursor),
                                                  gdk_cursor_get_hotspot_x (cursor),
                                                  gdk_cursor_get_hotspot_y (cursor));
+  else
+    {
+      int size;
+      GdkTexture *texture;
+      int width, height;
+      int hotspot_x, hotspot_y;
+
+      size = XcursorGetDefaultSize (GDK_DISPLAY_XDISPLAY (display));
+
+      texture = gdk_cursor_get_texture_for_size  (cursor, size, 1,
+                                                  &width, &height,
+                                                  &hotspot_x, &hotspot_y);
+
+      if (texture)
+        {
+          xcursor = gdk_x11_cursor_create_for_texture (display,
+                                                       texture,
+                                                       hotspot_x,
+                                                       hotspot_y);
+          g_object_unref (texture);
+        }
+    }
 
   if (xcursor != None)
     {
