@@ -15,6 +15,7 @@
 
 #include <glib/gstdio.h>
 
+#include "gdk/gdkdisplayprivate.h"
 #include "gdk/gdkdmabuftexturebuilderprivate.h"
 #include "gdk/gdkdmabuftextureprivate.h"
 #include "gdk/gdkglcontextprivate.h"
@@ -279,8 +280,7 @@ release_dmabuf_texture (gpointer data)
 {
   Texture *texture = data;
 
-  for (unsigned int i = 0; i < texture->dmabuf.n_planes; i++)
-    g_close (texture->dmabuf.planes[i].fd, NULL);
+  gdk_dmabuf_close_fds (&texture->dmabuf);
   g_free (texture);
 }
 #endif
@@ -296,8 +296,10 @@ gsk_gpu_download_op_gl_command (GskGpuOp          *op,
   GdkGLContext *context;
   guint texture_id;
 
-  context = GDK_GL_CONTEXT (gsk_gpu_frame_get_context (frame));
-  texture_id = gsk_gl_image_steal_texture (GSK_GL_IMAGE (self->image));
+  /* Don't use the renderer context, the texture might survive the frame
+   * and its surface */
+  context = gdk_display_get_gl_context (gsk_gpu_device_get_display (gsk_gpu_frame_get_device (frame)));
+  texture_id = gsk_gl_image_get_texture_id (GSK_GL_IMAGE (self->image));
 
 #ifdef HAVE_DMABUF
   if (self->allow_dmabuf)
@@ -323,6 +325,8 @@ gsk_gpu_download_op_gl_command (GskGpuOp          *op,
 
           if (self->texture)
             return op->next;
+
+          gdk_dmabuf_close_fds (&texture->dmabuf);
         }
 
       g_free (texture);
@@ -348,6 +352,9 @@ gsk_gpu_download_op_gl_command (GskGpuOp          *op,
                                                 gsk_gl_texture_data_free,
                                                 data);
 
+  gsk_gpu_image_toggle_ref_texture (self->image, self->texture);
+  gsk_gl_image_steal_texture_ownership (GSK_GL_IMAGE (self->image));
+
   g_object_unref (builder);
 
   return op->next;
