@@ -17,8 +17,10 @@ void                    gsk_gpu_print_newline                           (GString
 
 void                    gsk_gpu_print_string                            (GString                        *string,
                                                                          const char                     *s);
-void                    gsk_gpu_print_shader_info                       (GString                        *string,
-                                                                         GskGpuShaderClip                clip);
+void                    gsk_gpu_print_shader_flags                      (GString                        *string,
+                                                                         GskGpuShaderFlags               flags);
+void                    gsk_gpu_print_color_states                      (GString                        *string,
+                                                                         GskGpuColorStates               color_states);
 void                    gsk_gpu_print_enum                              (GString                        *string,
                                                                          GType                           type,
                                                                          int                             value);
@@ -32,6 +34,3 @@ void                    gsk_gpu_print_rgba                              (GString
                                                                          const float                     rgba[4]);
 void                    gsk_gpu_print_image                             (GString                        *string,
                                                                          GskGpuImage                    *image);
-void                    gsk_gpu_print_image_descriptor                  (GString                        *string,
-                                                                         GskGpuDescriptors              *desc,
-                                                                         guint32                         descriptor);
