@@ -29,7 +29,7 @@ G_BEGIN_DECLS
 
 #define GTK_TYPE_COLUMN_VIEW_ROW (gtk_column_view_row_get_type ())
 GDK_AVAILABLE_IN_4_12
-GDK_DECLARE_INTERNAL_TYPE(GtkColumnViewRow, gtk_column_view_row, GTK, COLUMN_VIEW_ROW, GObject);
+GDK_DECLARE_INTERNAL_TYPE(GtkColumnViewRow, gtk_column_view_row, GTK, COLUMN_VIEW_ROW, GObject)
 
 GDK_AVAILABLE_IN_4_12
 gpointer        gtk_column_view_row_get_item                    (GtkColumnViewRow       *self);
