@@ -52,6 +52,9 @@
 #define GDK_ARRAY_PREALLOC 64
 #include "gdk/gdkarrayimpl.c"
 
+/* For lack of a better place, assert here that these two definitions match */
+G_STATIC_ASSERT (GTK_DEBUG_CSS == GTK_CSS_PARSER_DEBUG_CSS);
+
 /**
  * GtkCssProvider:
  *
@@ -220,6 +223,9 @@ gtk_css_provider_class_init (GtkCssProviderClass *klass)
    *
    * Signals that a parsing error occurred.
    *
+   * The expected error values are in the [error@Gtk.CssParserError]
+   * and [enum@Gtk.CssParserWarning] enumerations.
+   *
    * The @path, @line and @position describe the actual location of
    * the error as accurately as possible.
    *
@@ -228,6 +234,9 @@ gtk_css_provider_class_init (GtkCssProviderClass *klass)
    * even all of it to not be parsed at all. So it is a useful idea
    * to check that the parsing succeeds by connecting to this signal.
    *
+   * Errors in the [enum@Gtk.CssParserWarning] enumeration should not
+   * be treated as fatal errors.
+   *
    * Note that this signal may be emitted at any time as the css provider
    * may opt to defer parsing parts or all of the input to a later time
    * than when a loading function was called.
