@@ -17,8 +17,7 @@
  *
  * Author(s): Carlos Garnacho <carlosg@gnome.org>
  */
-#ifndef __GTK_EVENT_CONTROLLER_H__
-#define __GTK_EVENT_CONTROLLER_H__
+#pragma once
 
 #if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
 #error "Only <gtk/gtk.h> can be included directly."
@@ -40,15 +39,13 @@ G_BEGIN_DECLS
 #define GTK_EVENT_CONTROLLER_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GTK_TYPE_EVENT_CONTROLLER, GtkEventControllerClass))
 
 
+
 GDK_AVAILABLE_IN_ALL
 GType        gtk_event_controller_get_type       (void) G_GNUC_CONST;
 
 GDK_AVAILABLE_IN_ALL
 GtkWidget  * gtk_event_controller_get_widget     (GtkEventController *controller);
 
-GDK_AVAILABLE_IN_ALL
-gboolean     gtk_event_controller_handle_event   (GtkEventController *controller,
-                                                  const GdkEvent     *event);
 GDK_AVAILABLE_IN_ALL
 void         gtk_event_controller_reset          (GtkEventController *controller);
 
@@ -70,7 +67,20 @@ const char *        gtk_event_controller_get_name              (GtkEventControll
 GDK_AVAILABLE_IN_ALL
 void                gtk_event_controller_set_name              (GtkEventController *controller,
                                                                 const char         *name);
+GDK_AVAILABLE_IN_4_8
+void                gtk_event_controller_set_static_name       (GtkEventController *controller,
+                                                                const char         *name);
+
+GDK_AVAILABLE_IN_ALL
+GdkEvent *          gtk_event_controller_get_current_event    (GtkEventController *controller);
+GDK_AVAILABLE_IN_ALL
+guint32             gtk_event_controller_get_current_event_time   (GtkEventController *controller);
+GDK_AVAILABLE_IN_ALL
+GdkDevice *         gtk_event_controller_get_current_event_device (GtkEventController *controller);
+GDK_AVAILABLE_IN_ALL
+GdkModifierType     gtk_event_controller_get_current_event_state (GtkEventController *controller);
+
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GtkEventController, g_object_unref)
 
 G_END_DECLS
 
-#endif /* __GTK_EVENT_CONTROLLER_H__ */
