@@ -24,7 +24,6 @@
 
 #include "gtkrevealer.h"
 
-#include "gtkintl.h"
 #include "gtkprivate.h"
 #include "gtkprogresstrackerprivate.h"
 #include "gtksettingsprivate.h"
@@ -315,9 +314,7 @@ gtk_revealer_class_init (GtkRevealerClass *klass)
    * The type of animation used to transition.
    */
   props[PROP_TRANSITION_TYPE] =
-    g_param_spec_enum ("transition-type",
-                       P_("Transition type"),
-                       P_("The type of animation used to transition"),
+    g_param_spec_enum ("transition-type", NULL, NULL,
                        GTK_TYPE_REVEALER_TRANSITION_TYPE,
                        GTK_REVEALER_TRANSITION_TYPE_SLIDE_DOWN,
                        GTK_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_EXPLICIT_NOTIFY);
@@ -328,21 +325,17 @@ gtk_revealer_class_init (GtkRevealerClass *klass)
    * The animation duration, in milliseconds.
    */
   props[PROP_TRANSITION_DURATION] =
-    g_param_spec_uint ("transition-duration",
-                       P_("Transition duration"),
-                       P_("The animation duration, in milliseconds"),
+    g_param_spec_uint ("transition-duration", NULL, NULL,
                        0, G_MAXUINT, 250,
                        GTK_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkRevealer:reveal-child: (attributes org.gtk.Proeprty.get=gtk_revealer_get_reveal_child org.gtk.Property.set=gtk_revealer_set_reveal_child)
+   * GtkRevealer:reveal-child: (attributes org.gtk.Property.get=gtk_revealer_get_reveal_child org.gtk.Property.set=gtk_revealer_set_reveal_child)
    *
    * Whether the revealer should reveal the child.
    */
   props[PROP_REVEAL_CHILD] =
-    g_param_spec_boolean ("reveal-child",
-                          P_("Reveal Child"),
-                          P_("Whether the container should reveal the child"),
+    g_param_spec_boolean ("reveal-child", NULL, NULL,
                           FALSE,
                           GTK_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_EXPLICIT_NOTIFY);
 
@@ -352,9 +345,7 @@ gtk_revealer_class_init (GtkRevealerClass *klass)
    * Whether the child is revealed and the animation target reached.
    */
   props[PROP_CHILD_REVEALED] =
-    g_param_spec_boolean ("child-revealed",
-                          P_("Child Revealed"),
-                          P_("Whether the child is revealed and the animation target reached"),
+    g_param_spec_boolean ("child-revealed", NULL, NULL,
                           FALSE,
                           GTK_PARAM_READABLE);
 
@@ -364,9 +355,7 @@ gtk_revealer_class_init (GtkRevealerClass *klass)
    * The child widget.
    */
   props[PROP_CHILD] =
-    g_param_spec_object ("child",
-                         P_("Child"),
-                         P_("The child widget"),
+    g_param_spec_object ("child", NULL, NULL,
                          GTK_TYPE_WIDGET,
                          GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
@@ -866,7 +855,7 @@ gtk_revealer_set_transition_type (GtkRevealer               *revealer,
 /**
  * gtk_revealer_set_child: (attributes org.gtk.Method.set_property=child)
  * @revealer: a `GtkRevealer`
- * @child: (allow-none): the child widget
+ * @child: (nullable): the child widget
  *
  * Sets the child widget of @revealer.
  */
@@ -875,7 +864,10 @@ gtk_revealer_set_child (GtkRevealer *revealer,
                         GtkWidget   *child)
 {
   g_return_if_fail (GTK_IS_REVEALER (revealer));
-  g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
+  g_return_if_fail (child == NULL || revealer->child == child || gtk_widget_get_parent (child) == NULL);
+
+  if (revealer->child == child)
+    return;
 
   g_clear_pointer (&revealer->child, gtk_widget_unparent);
 
