@@ -101,8 +101,15 @@ static GParamSpec *properties[NUM_PROPERTIES] = { NULL, };
 static GtkTreeListModel *
 tree_node_get_tree_list_model (TreeNode *node)
 {
-  for (; !node->is_root; node = node->parent)
-    { }
+  if (node->is_root)
+    return node->list;
+
+  for (node = node->parent; !node->is_root; node = node->parent)
+    {
+      /* This can happen during collapsing of a parent node */
+      if (node->children == NULL)
+        return NULL;
+    }
 
   return node->list;
 }
@@ -168,7 +175,7 @@ tree_node_get_position (TreeNode *node)
   TreeNode *left, *parent;
   TreeAugment *left_aug;
   guint n;
-  
+
   for (n = 0;
        !node->is_root;
        node = node->parent, n++)
@@ -316,6 +323,9 @@ gtk_tree_list_model_items_changed_cb (GListModel *model,
   guint i, tree_position, tree_removed, tree_added, n_local;
 
   self = tree_node_get_tree_list_model (node);
+  if (self == NULL)
+    return;
+
   n_local = g_list_model_get_n_items (model) - added + removed;
 
   if (position < n_local)
@@ -357,7 +367,7 @@ gtk_tree_list_model_items_changed_cb (GListModel *model,
     }
 
   tree_added = added;
-  for (i = 0; i < added; i++)
+  for (i = added; i-- > 0;)
     {
       child = gtk_rb_tree_insert_before (node->children, child);
       child->parent = node;
@@ -405,10 +415,16 @@ gtk_tree_list_model_clear_node (gpointer data)
   TreeNode *node = data;
 
   if (node->row)
-    gtk_tree_list_row_destroy (node->row);
+    {
+      g_object_freeze_notify (G_OBJECT (node->row));
+      gtk_tree_list_row_destroy (node->row);
+    }
 
   gtk_tree_list_model_clear_node_children (node);
 
+  if (node->row)
+    g_object_thaw_notify (G_OBJECT (node->row));
+
   g_clear_object (&node->item);
 }
 
@@ -479,7 +495,7 @@ gtk_tree_list_model_expand_node (GtkTreeListModel *self,
 
   if (node->empty)
     return 0;
-  
+
   if (node->model != NULL)
     return 0;
 
@@ -487,18 +503,18 @@ gtk_tree_list_model_expand_node (GtkTreeListModel *self,
 
   if (model == NULL)
     return 0;
-  
+
   gtk_tree_list_model_init_node (self, node, model);
 
   tree_node_mark_dirty (node);
-  
+
   return tree_node_get_n_children (node);
 }
 
 static guint
 gtk_tree_list_model_collapse_node (GtkTreeListModel *self,
                                    TreeNode         *node)
-{      
+{
   guint n_items;
 
   if (node->model == NULL)
@@ -589,7 +605,7 @@ gtk_tree_list_model_set_property (GObject      *object,
     }
 }
 
-static void 
+static void
 gtk_tree_list_model_get_property (GObject     *object,
                                   guint        prop_id,
                                   GValue      *value,
@@ -719,9 +735,9 @@ gtk_tree_list_model_init (GtkTreeListModel *self)
  * @root: (transfer full): The `GListModel` to use as root
  * @passthrough: %TRUE to pass through items from the models
  * @autoexpand: %TRUE to set the autoexpand property and expand the @root model
- * @create_func: Function to call to create the `GListModel` for the children
- *   of an item
- * @user_data: (closure): Data to pass to @create_func
+ * @create_func: (scope notified) (closure user_data) (destroy user_destroy): function to
+ *   call to create the `GListModel` for the children of an item
+ * @user_data: Data to pass to @create_func
  * @user_destroy: Function to call to free @user_data
  *
  * Creates a new empty `GtkTreeListModel` displaying @root
@@ -945,16 +961,12 @@ G_DEFINE_TYPE (GtkTreeListRow, gtk_tree_list_row, G_TYPE_OBJECT)
 static void
 gtk_tree_list_row_destroy (GtkTreeListRow *self)
 {
-  g_object_freeze_notify (G_OBJECT (self));
-
   self->node = NULL;
 
   /* FIXME: We could check some properties to avoid excess notifies */
   g_object_notify_by_pspec (G_OBJECT (self), row_properties[ROW_PROP_DEPTH]);
   g_object_notify_by_pspec (G_OBJECT (self), row_properties[ROW_PROP_EXPANDABLE]);
   g_object_notify_by_pspec (G_OBJECT (self), row_properties[ROW_PROP_EXPANDED]);
-
-  g_object_thaw_notify (G_OBJECT (self));
 }
 
 static void
@@ -977,7 +989,7 @@ gtk_tree_list_row_set_property (GObject      *object,
     }
 }
 
-static void 
+static void
 gtk_tree_list_row_get_property (GObject     *object,
                                 guint        prop_id,
                                 GValue      *value,
@@ -1180,6 +1192,8 @@ gtk_tree_list_row_set_expanded (GtkTreeListRow *self,
     return;
 
   list = tree_node_get_tree_list_model (self->node);
+  if (list == NULL)
+    return;
 
   if (expanded)
     {
@@ -1255,6 +1269,9 @@ gtk_tree_list_row_is_expandable (GtkTreeListRow *self)
     return TRUE;
 
   list = tree_node_get_tree_list_model (self->node);
+  if (list == NULL)
+    return FALSE;
+
   model = tree_node_create_model (list, self->node);
   if (model)
     {
